package com.farmnet365.hello365.clients;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */


import com.farmnet365.hello365.model.field.Field;
import com.farmnet365.hello365.model.machine.Machine;
import com.farmnet365.hello365.model.machine.MachineResponse;
import com.farmnet365.hello365.model.person.Person;
import com.farmnet365.hello365.model.person.PersonResponse;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.List;

@Service
public class RestClient {

    /**
     * Get List of Fields
     *
     * @param apiBase      current api-base
     * @param partnerToken current partner-token
     * @return List<Field>
     */
    public List<Field> getFields(String apiBase, String partnerToken) {
        RestTemplate restTemplate = new RestTemplate();

        ResponseEntity<List<Field>> response = restTemplate.exchange(
                apiBase + RestConstants.FIELDS_PATH,
                HttpMethod.GET,
                buildHttpHeader(partnerToken),
                new ParameterizedTypeReference<>() {
                });

        return response.getBody();
    }

    /**
     * Get List of persons
     *
     * @param apiBase      current api-base
     * @param partnerToken current partner-token
     * @return PersonResponse
     */
    public PersonResponse getPersons(String apiBase, String partnerToken) {
        RestTemplate restTemplate = new RestTemplate();

        ResponseEntity<PersonResponse> response = restTemplate.exchange(
                apiBase + RestConstants.PERSONS_PATH,
                HttpMethod.GET,
                buildHttpHeader(partnerToken),
                new ParameterizedTypeReference<>() {
                });

        return response.getBody();
    }

    /**
     * Post a new person
     *
     * @param apiBase      current api-base
     * @param partnerToken current partner-token
     * @param person       new person
     * @return Person
     */
    public Person postPerson(String apiBase, String partnerToken, Person person) {
        RestTemplate restTemplate = new RestTemplate();

        HttpEntity<Person> request = new HttpEntity<>(person, buildHttpHeaderForPost(partnerToken));

        ResponseEntity<Person> response = restTemplate.exchange(
                apiBase + RestConstants.PERSONS_PATH,
                HttpMethod.POST,
                request,
                Person.class);

        return response.getBody();
    }

    /**
     * Get List of machines
     *
     * @param apiBase      current api-base
     * @param partnerToken current partner-token
     * @return MachineResponse
     */
    public MachineResponse getMachines(String apiBase, String partnerToken) {
        RestTemplate restTemplate = new RestTemplate();

        ResponseEntity<MachineResponse> response = restTemplate.exchange(
                apiBase + RestConstants.MACHINES_PATH,
                HttpMethod.GET,
                buildHttpHeader(partnerToken),
                new ParameterizedTypeReference<>() {
                });

        return response.getBody();
    }

    /**
     * Post a new machine
     *
     * @param apiBase      current api-base
     * @param partnerToken current partner-token
     * @param machine      new Machine
     * @return Machine
     */
    public Machine postMachine(String apiBase, String partnerToken, Machine machine) {
        RestTemplate restTemplate = new RestTemplate();

        HttpEntity<Machine> request = new HttpEntity<>(machine, buildHttpHeaderForPost(partnerToken));

        ResponseEntity<Machine> response = restTemplate.exchange(
                apiBase + RestConstants.MACHINES_PATH,
                HttpMethod.POST,
                request,
                Machine.class);

        return response.getBody();
    }

    /**
     * Builds http header with Authentication
     *
     * @param partnerToken the Authentication JSON web token
     * @return http header with Authentication
     */
    private HttpEntity<String> buildHttpHeader(String partnerToken) {
        HttpHeaders headers = new HttpHeaders();
        headers.set(HttpHeaders.AUTHORIZATION, partnerToken);
        return new HttpEntity<>("parameters", headers);
    }

    /**
     * Builds http header for post with Authentication
     *
     * @param partnerToken the Authentication JSON web token
     * @return http header with Authentication
     */
    private HttpHeaders buildHttpHeaderForPost(String partnerToken) {
        HttpHeaders headers = new HttpHeaders();
        headers.set(HttpHeaders.AUTHORIZATION, partnerToken);
        headers.set(HttpHeaders.CONTENT_TYPE, "application/json");
        headers.set(HttpHeaders.ACCEPT, "application/json");
        return headers;
    }

}
