package com.farmnet365.hello365.config;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */


import com.auth0.jwt.exceptions.SignatureVerificationException;
import com.farmnet365.hello365.exception.JwtVerifyException;
import com.farmnet365.hello365.model.ErrorDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.client.HttpClientErrorException;

@ControllerAdvice
public class GlobalExceptionHandler {

    private static final Logger log = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    public static final String ERROR_CODE_0001 = "0001_UnexpectedError";
    public static final String ERROR_CODE_0002 = "0002_JwtVerifyError";
    public static final String ERROR_CODE_0003 = "0003_HttpClientError";
    public static final String ERROR_CODE_0004 = "0004_SignatureVerificationException";

    /**
     * JWT verify exception handler.
     *
     * @param ex the thrown jwtVerifyException exception
     * @return an ErrorDto
     */
    @ExceptionHandler(JwtVerifyException.class)
    public ResponseEntity<ErrorDto> jwtVerifyException(JwtVerifyException ex) {
        log.error("An JwtVerifyException occured in hello365: {}", ex.getMessage(), ex);

        var errorDto = new ErrorDto();
        errorDto.setCode(ERROR_CODE_0002);
        errorDto.setMessage(ex.getMessage());

        return new ResponseEntity<>(errorDto, HttpStatus.UNAUTHORIZED);
    }

    /**
     * 365Connect error exception handler.
     *
     * @param ex the thrown jwtVerifyException exception
     * @return an ErrorDto
     */
    @ExceptionHandler(HttpClientErrorException.class)
    public ResponseEntity<ErrorDto> httpClientErrorException(HttpClientErrorException ex) {
        log.error("An HttpClientErrorException occured in hello365: {}", ex.getMessage(), ex);

        var errorDto = new ErrorDto();
        errorDto.setCode(ERROR_CODE_0003);
        errorDto.setMessage(ex.getMessage());

        return new ResponseEntity<>(errorDto, HttpStatus.INTERNAL_SERVER_ERROR);
    }

    /**
     *  SignatureVerificationException
     *
     * @param ex the thrown exception
     * @return an ErrorDto
     */
    @ExceptionHandler(SignatureVerificationException.class)
    public ResponseEntity<ErrorDto> signatureVerificationException(SignatureVerificationException ex) {
        log.error("An SignatureVerificationException occured in hello365: {}", ex.getMessage(), ex);

        var errorDto = new ErrorDto();
        errorDto.setCode(ERROR_CODE_0004);
        errorDto.setMessage(ex.getMessage());

        return new ResponseEntity<>(errorDto, HttpStatus.UNAUTHORIZED);
    }

    /**
     * General exception handler.
     *
     * @param ex the thrown exception
     * @return an ErrorDto
     */
    @ExceptionHandler(Throwable.class)
    public ResponseEntity<ErrorDto> handleException(Throwable ex) {
        log.error("An exception occured in hello365: {}", ex.getMessage(), ex);

        var errorDto = new ErrorDto();
        errorDto.setCode(ERROR_CODE_0001);
        errorDto.setMessage(ex.getMessage());

        return new ResponseEntity<>(errorDto, HttpStatus.INTERNAL_SERVER_ERROR);
    }

}
