package com.farmnet365.hello365.controller;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */


import com.farmnet365.hello365.model.field.Field;
import com.farmnet365.hello365.model.machine.Machine;
import com.farmnet365.hello365.model.machine.MachineResponse;
import com.farmnet365.hello365.model.person.Person;
import com.farmnet365.hello365.model.person.PersonResponse;
import com.farmnet365.hello365.service.ConnectService;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import java.net.URI;
import java.util.List;

@RestController
@RequestMapping(path = "/rest")
public class Hello365Controller {

    private static final Logger log = org.slf4j.LoggerFactory.getLogger(Hello365Controller.class);
    private final ConnectService connect365Service;

	@Autowired
	public Hello365Controller(final ConnectService connect365Service) {
		this.connect365Service = connect365Service;
	}


	/**
	 * GET fields REST endpoint.
	 *
	 * @param auth
	 *            JWT authentication token from 365Connect API
	 * @return List of fields
	 */
	@RequestMapping(path = "/fields", method = RequestMethod.GET)
	public ResponseEntity<List<Field>> getFields(@RequestHeader(value = HttpHeaders.AUTHORIZATION) String auth) {
		log.debug("---->> getFields auth: {}", auth);
		List<Field> fields = connect365Service.getFields(auth);
		log.debug("<<---- getFields return: {}", fields.toString());
		return ResponseEntity.ok(fields);
	}

	/**
	 * GET persons REST endpoint.
	 *
	 * @param auth
	 *            JWT authentication token from 365Connect API
	 * @return List of persons
	 */
	@RequestMapping(path = "/persons", method = RequestMethod.GET)
	public ResponseEntity<PersonResponse> getPersons(@RequestHeader(value = HttpHeaders.AUTHORIZATION) String auth) {
		log.debug("---->> getPersons auth: {}", auth);
		PersonResponse persons = connect365Service.getPersons(auth);
		log.debug("<<---- getPersons return: {}", persons.toString());
		return ResponseEntity.ok(persons);
	}

	/**
	 * POST person REST endpoint.
	 *
	 * @param auth
	 *            JWT authentication token from 365Connect API
	 * @param person
	 *            POST request body
	 * @return List of persons
	 */
	@RequestMapping(path = "/persons", method = RequestMethod.POST)
	public ResponseEntity<?> createPerson(@RequestHeader(value = HttpHeaders.AUTHORIZATION) String auth, @RequestBody Person person) {
		log.debug("---->> createPerson : {}", person);
		if (person == null) {
			return ResponseEntity.badRequest().build();
		}

		URI location = ServletUriComponentsBuilder
				.fromCurrentRequest()
				.path("/{id}")
				.buildAndExpand(connect365Service.createPerson(person, auth).getId())
				.toUri();

		log.debug("<<---- createPerson return.");
		return ResponseEntity.created(location).build();
	}


	/**
	 * GET persons REST endpoint.
	 *
	 * @param auth JWT authentication token from 365Connect API
	 * @return List of machines
	 */
	@RequestMapping(path = "/machines", method = RequestMethod.GET)
	public ResponseEntity<MachineResponse> getMachines(@RequestHeader(value = HttpHeaders.AUTHORIZATION) String auth) {
		log.debug("---->> getMachines auth: {}", auth);
		MachineResponse machines = connect365Service.getMachines(auth);
		log.debug("<<---- getMachines return: {}", machines.toString());
		return ResponseEntity.ok(machines);
	}


	@RequestMapping(path = "/machines", method = RequestMethod.POST)
	public ResponseEntity<?> createMachine(@RequestHeader(value = HttpHeaders.AUTHORIZATION) String auth, @RequestBody Machine machine) {
		log.debug("---->> createMachine : {}", machine);
		if (machine == null) {
			return ResponseEntity.badRequest().build();
		}

		URI location = ServletUriComponentsBuilder
				.fromCurrentRequest()
				.path("/{id}")
				.buildAndExpand(connect365Service.createMachine(machine, auth).getId())
				.toUri();

		log.debug("<<---- createMachine return.");
		return ResponseEntity.created(location).build();
	}
}
