package com.farmnet365.hello365.service;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */


import com.farmnet365.hello365.model.field.Field;
import com.farmnet365.hello365.model.machine.Machine;
import com.farmnet365.hello365.model.machine.MachineResponse;
import com.farmnet365.hello365.model.person.Person;
import com.farmnet365.hello365.model.person.PersonResponse;
import com.farmnet365.hello365.clients.RestClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;

/**
 * Service to communicate with 365Connect REST API.
 */
@Service
public class ConnectService {

    private static final Logger log = LoggerFactory.getLogger(ConnectService.class);

	private final JwtService jwtService;
	private final RestClient restClient;

	@Autowired
	public ConnectService(final RestClient restClient, final JwtService jwtService) {
		this.restClient = restClient;
		this.jwtService = jwtService;
	}

	/**
	 * Calls GET on the 365Connect API on entity fields
	 *
	 * @param auth The Authentication JSON web token
	 * @return List of fields
	 */
	public List<Field> getFields(String auth) {
		log.info("start getFields.");

		ConnectApiAuthenticationInfo generatedAuth = jwtService.generatePartnerJwt(auth);
		List<Field> results = restClient.getFields(generatedAuth.getApiBase(), generatedAuth.getPartnerToken());

		// Sort fields by number.
		results.sort(Comparator.comparing(Field::getNumber));
		log.info("end getFields: {} found.", results.size());
		return results;
	}

	/**
	 * Calls GET on the 365Connect API on entity persons
	 *
	 * @param auth The Authentication JSON web token
	 * @return List of persons
	 */
	public PersonResponse getPersons(String auth) {
		log.info("start getPersons.");

		ConnectApiAuthenticationInfo generatedAuth = jwtService.generatePartnerJwt(auth);
		PersonResponse results = restClient.getPersons(generatedAuth.getApiBase(), generatedAuth.getPartnerToken());

		// Sort results by name desc.
		results.getList().sort(Comparator.comparing(Person::getName).reversed());
		log.info("end getPersons: {} found.", results.getList().size());
		return results;
	}

	/**
	 * Creates a new person entity via 365Connect.
	 *
	 * @param body new person
	 * @param auth The Authentication JSON web token
	 * @return URL of new person
	 */
	public Person createPerson(Person body, String auth) {
		log.info("start createPerson.");

		ConnectApiAuthenticationInfo generatedAuth = jwtService.generatePartnerJwt(auth);

		Person result = restClient.postPerson(generatedAuth.getApiBase(), generatedAuth.getPartnerToken(), body);
		log.info("end createPerson.");
		return result;
	}

	/**
	 * Calls GET on the 365Connect API on entity machines
	 *
	 * @param auth The Authentication JSON web token
	 * @return List of machines
	 */
	public MachineResponse getMachines(String auth) {
		log.info("start getMachines.");

		ConnectApiAuthenticationInfo generatedAuth = jwtService.generatePartnerJwt(auth);

		MachineResponse results = restClient.getMachines(generatedAuth.getApiBase(), generatedAuth.getPartnerToken());
		log.info("end getMachines: {} found.", results.getList().size());
		return results;
	}

	/**
	 * Creates a new Machine entity via 365Connect.
	 *
	 * @param machine new machine
	 * @param auth The Authentication JSON web token
	 * @return URL of new person
	 */
	public Machine createMachine(Machine machine, String auth) {
		log.info("start createMachine.");

		ConnectApiAuthenticationInfo generatedAuth = jwtService.generatePartnerJwt(auth);

		Machine result = restClient.postMachine(generatedAuth.getApiBase(), generatedAuth.getPartnerToken(), machine);
		log.info("end createMachine.");
		return result;
	}
}
