package com.farmnet365.hello365.service;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */


import com.auth0.jwt.JWT;
import com.auth0.jwt.exceptions.JWTCreationException;
import com.auth0.jwt.exceptions.JWTDecodeException;
import com.auth0.jwt.interfaces.Claim;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.farmnet365.hello365.config.ApiProperties;
import com.farmnet365.hello365.exception.JwtVerifyException;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * This service bean verifies the 365Connect JSON web token (jwt) and generates a partner jwt to authenticate
 * to the 365Connect API.
 */
@Service
public class JwtService {

    private static final Logger log = LoggerFactory.getLogger(JwtService.class);

    private static final String FN_EXT = "fn-ext";
    private static final String API_BASE = "apiBase";
    private static final String CON_CLAIM = "con";
    private static final String BEARER = "Bearer ";

    private ApiProperties apiProperties;
    private SigningAlgorithmService signingAlgorithmService;

    @Autowired
    public JwtService(ApiProperties apiProperties, SigningAlgorithmService signingAlgorithmService) {
        this.apiProperties = apiProperties;
        this.signingAlgorithmService = signingAlgorithmService;
    }


    /**
     * Generates the 365Partner jwt.
     *
     * @param connectJwt the 365 connect jwt
     * @return ConnectApiAuthentication object containing the partner jwt and the api base url.
     * @throws JwtVerifyException
     */
    public ConnectApiAuthenticationInfo generatePartnerJwt(String connectJwt) {

        // 1. decode incoming connect-token
        DecodedJWT decodedConnectJwt = decodeConnectJwt(connectJwt);

        // 2. verify if connect-token is valid
        verifyConnectJwt(decodedConnectJwt);

        // 3. extract our current apiBase
        String apiBase = extractApiBase(decodedConnectJwt);

        // 4. create new Partner Token
        ConnectApiAuthenticationInfo partnerToken = createPartnerJwt(connectJwt, decodedConnectJwt, apiBase);

        return partnerToken;
    }

    /**
     * extract our current Api-Base out of the connect-token
     *
     * @param decodedConnectJwt connect-token
     * @return apiBase
     */
    protected String extractApiBase(DecodedJWT decodedConnectJwt) {
        Claim fnExtClaim = decodedConnectJwt.getClaim(FN_EXT);
        String apiBase = (String) fnExtClaim.asMap().get(API_BASE);
        log.debug("apiBase found: {}", apiBase);

        return apiBase;
    }

    /**
     * Creates a new partner-token
     *
     * @param connectJwt        the original connectJwt
     * @param decodedConnectJwt the decoded connectJwt
     * @param apiBase           the apiBase
     * @return ConnectApiAuthenticationInfo
     */
    protected ConnectApiAuthenticationInfo createPartnerJwt(String connectJwt, DecodedJWT decodedConnectJwt, String apiBase) {

        try {
            String partnerToken = JWT
                    .create()
                    .withHeader(generatePartnerJwtHeader())
                    .withIssuer(apiProperties.getId())
                    .withExpiresAt(decodedConnectJwt.getExpiresAt())
                    .withIssuedAt(decodedConnectJwt.getIssuedAt())
                    .withClaim(CON_CLAIM, connectJwt)
                    .sign(signingAlgorithmService.getPartnerTokenAlgorithm());

            return new ConnectApiAuthenticationInfo(BEARER + partnerToken, apiBase);

        } catch (JWTCreationException ex) {
            throw new JwtVerifyException("Unable to create partner jwt. Invalid signing configuration. Unable to convert Claims.", ex);
        }
    }

    /**
     * Decodes the connect jwt.
     *
     * @param connectJwt the connect jwt to decode.
     * @return the decoded jwt.
     * @throws JwtVerifyException
     */
    protected DecodedJWT decodeConnectJwt(String connectJwt) {
        if (connectJwt == null) {
            throw new JwtVerifyException("No connect jwt present.");
        }
        try {
            return JWT.decode(connectJwt);
        } catch (JWTDecodeException ex) {
            throw new JwtVerifyException("Unable to decode connect jwt.", ex);
        }
    }

    /**
     * Verifies following connect jwt claims: - validates the expiration date - the issued date - checks the presents of an API
     * base URL
     *
     * @param jwt the decoded connect jwt
     * @throws JwtVerifyException
     */
    protected void verifyConnectJwt(DecodedJWT jwt) {

        // Check whether the token was signed correctly by 365FarmNet.
        signingAlgorithmService.verifyConnectToken(jwt);
        log.debug("token was signed correctly.");

        if (!Optional.ofNullable(jwt.getExpiresAt()).map(d -> d.after(new Date())).orElse(false)) {
            throw new JwtVerifyException("Unable to create partner jwt. Connect jwt is expired.");
        }
        log.debug("expiredAt is valid.");

        if (!Optional.ofNullable(jwt.getIssuedAt()).map(d -> d.before(new Date())).orElse(false)) {
            throw new JwtVerifyException("Unable to create partner jwt. Connect jwt issue date is invalid.");
        }
        log.debug("issuedAt is valid.");

        // check if 'fn-ext' exists
        Claim fnExtClaim = jwt.getClaim(FN_EXT);
        if (fnExtClaim.isNull()) {
            throw new JwtVerifyException("Unable to create partner jwt. No fn-ext found in Claims.");
        }
        log.debug("fn-ext exists.");

        // check if the claim 'apiBase' exists
        if (fnExtClaim.asMap() == null ||
                !fnExtClaim.asMap().containsKey(API_BASE) ||
                StringUtils.isEmpty((CharSequence) fnExtClaim.asMap().get(API_BASE))) {
            throw new JwtVerifyException("Unable to create partner jwt. No apiBase found in Claims.");
        }
        log.debug("apiBase exists.");
    }

    private Map<String, Object> generatePartnerJwtHeader() {
        Map<String, Object> jwtHeader = new HashMap<>();
        jwtHeader.put("ver", "0.1");
        jwtHeader.put("type", "partner");
        jwtHeader.put("alg", "HS256");
        return jwtHeader;
    }

}

