package com.farmnet365.hello365.service;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */


import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.farmnet365.hello365.config.ApiProperties;
import com.farmnet365.hello365.exception.JwtVerifyException;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.util.io.pem.PemObject;
import org.bouncycastle.util.io.pem.PemReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.security.KeyFactory;
import java.security.PublicKey;
import java.security.Security;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.X509EncodedKeySpec;

/**
 * The primary goal of this class is to keep the algorithms for the JWT signature in
 * order to achieve a better testability.
 *
 * In addition, the public key from the PEM file is read here.
 */
@Service
public class SigningAlgorithmService {

    private static final Logger log = LoggerFactory.getLogger(SigningAlgorithmService.class);

    private Algorithm connectTokenAlgorithm;
    private Algorithm partnerTokenAlgorithm;

    private ApiProperties apiProperties;

    @Autowired
    public SigningAlgorithmService(ApiProperties apiProperties) {
        this.apiProperties = apiProperties;
    }

    @PostConstruct
    protected void setup() {
        // setup our signing algorithms
        PublicKey publicKey = encodePublicKey();
        if (publicKey == null) {
            throw new JwtVerifyException("365FarmNet public key is missing.");
        }
        connectTokenAlgorithm = Algorithm.RSA256((RSAPublicKey) publicKey, null);
        partnerTokenAlgorithm = Algorithm.HMAC256(apiProperties.getSecret());
        log.debug("Setup complete.");
    }

    /**
     * Read and encode the 365farmnet public key
     *
     * @return PublicKey
     */
    protected PublicKey encodePublicKey() {

        try {
            PemObject pemObject = readPemFile();
            X509EncodedKeySpec pubKeySpec = new X509EncodedKeySpec(pemObject.getContent());
            KeyFactory factory = KeyFactory.getInstance("RSA", "BC");
            PublicKey key = factory.generatePublic(pubKeySpec);
            log.debug("Key: {}", key);
            return key;

        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return null;
        }
    }

    /**
     * Read the provided public key
     *
     * @return PemObject
     * @throws IOException
     */
    protected PemObject readPemFile() throws IOException {

        Security.addProvider(new BouncyCastleProvider());
        InputStream resource = new ClassPathResource(apiProperties.getPublicKeyFile()).getInputStream();

        try (PemReader pemReader = new PemReader(new InputStreamReader(resource))) {
            return pemReader.readPemObject();
        }
    }

    public void verifyConnectToken(DecodedJWT jwt) {
        connectTokenAlgorithm.verify(jwt);
    }

    public Algorithm getPartnerTokenAlgorithm() {
        return partnerTokenAlgorithm;
    }

}
