package com.farmnet365.hello365.controller;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */


import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.SignatureVerificationException;
import com.farmnet365.hello365.config.GlobalExceptionHandler;
import com.farmnet365.hello365.exception.JwtVerifyException;
import com.farmnet365.hello365.model.field.Field;
import com.farmnet365.hello365.model.machine.Machine;
import com.farmnet365.hello365.model.machine.MachineResponse;
import com.farmnet365.hello365.model.person.Person;
import com.farmnet365.hello365.model.person.PersonResponse;
import com.farmnet365.hello365.service.ConnectService;
import com.google.gson.Gson;
import org.hamcrest.core.Is;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.web.client.HttpClientErrorException;

import java.security.interfaces.RSAPublicKey;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

@RunWith(SpringRunner.class)
@WebMvcTest(Hello365Controller.class)
public class Hello365ControllerTest {

    private static final String TEST_HEADER = "HGHFDZEJJDFJKWOIFMVNGJKDLHFJGDSZFGe673UEH";

    @Autowired
    private MockMvc mockMvc;

    @Mock
    private RSAPublicKey rsaPublicKey;

    @MockBean
    private ConnectService connectService;

    @Test
    public void getFieldsTest() throws Exception {

        Field field1 = new Field();
        field1.setId("1");
        List<Field> testResult = List.of(field1);

        when(connectService.getFields(anyString())).thenReturn(testResult);

        mockMvc.perform(get("/rest/fields").header(HttpHeaders.AUTHORIZATION, TEST_HEADER))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$[0].id", Is.is("1")));
    }

    @Test
    public void getPersonTest() throws Exception {

        Person person1 = new Person();
        person1.setId("1");
        PersonResponse response = new PersonResponse(List.of(person1));

        when(connectService.getPersons(anyString())).thenReturn(response);

        mockMvc.perform(get("/rest/persons").header(HttpHeaders.AUTHORIZATION, TEST_HEADER))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.list[0].id", Is.is("1")));
    }

    @Test
    public void getPersonVerifyFailTest() throws Exception {

        when(connectService.getPersons(anyString())).thenThrow(new JwtVerifyException("Connect jwt is expired."));

        mockMvc.perform(get("/rest/persons").header(HttpHeaders.AUTHORIZATION, TEST_HEADER))
                .andDo(print())
                .andExpect(status().isUnauthorized())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.code", Is.is(GlobalExceptionHandler.ERROR_CODE_0002)))
                .andExpect(jsonPath("$.message", Is.is("Connect jwt is expired.")));
    }

    @Test
    public void getMaschinesTest() throws Exception {

        Machine machine = new Machine();
        machine.setId("7532");
        MachineResponse response = new MachineResponse(List.of(machine));

        when(connectService.getMachines(anyString())).thenReturn(response);

        mockMvc.perform(get("/rest/machines").header(HttpHeaders.AUTHORIZATION, TEST_HEADER))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.list[0].id", Is.is("7532")));
    }

    @Test
    public void getMaschinesFailedSignatureTest() throws Exception {

        when(connectService.getMachines(anyString())).thenThrow(new SignatureVerificationException(Algorithm.RSA256(rsaPublicKey, null)));

        mockMvc.perform(get("/rest/machines").header(HttpHeaders.AUTHORIZATION, TEST_HEADER))
                .andDo(print())
                .andExpect(status().isUnauthorized())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.message").isNotEmpty())
                .andExpect(jsonPath("$.code", Is.is(GlobalExceptionHandler.ERROR_CODE_0004)));
    }

    @Test
    public void getMaschinesFailedHttpClientTest() throws Exception {

        when(connectService.getMachines(anyString())).thenThrow(new HttpClientErrorException(HttpStatus.I_AM_A_TEAPOT));

        mockMvc.perform(get("/rest/machines").header(HttpHeaders.AUTHORIZATION, TEST_HEADER))
                .andDo(print())
                .andExpect(status().isInternalServerError())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.message").isNotEmpty())
                .andExpect(jsonPath("$.code", Is.is(GlobalExceptionHandler.ERROR_CODE_0003)));
    }

    @Test
    public void getMaschinesFailedInternalServerErrorTest() throws Exception {

        when(connectService.getMachines(anyString())).thenThrow(new RuntimeException(("Burning...")));

        mockMvc.perform(get("/rest/machines").header(HttpHeaders.AUTHORIZATION, TEST_HEADER))
                .andDo(print())
                .andExpect(status().isInternalServerError())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON))
                .andExpect(jsonPath("$.message").isNotEmpty())
                .andExpect(jsonPath("$.code", Is.is(GlobalExceptionHandler.ERROR_CODE_0001)));
    }

    @Test
    public void postMaschinesRequestTest() throws Exception {
        Machine newMachine = new Machine();
        newMachine.setName("SchwartenLeger");
        Gson gson = new Gson();

        when(connectService.createMachine(any(Machine.class), anyString())).thenReturn(newMachine);
        ArgumentCaptor<Machine> machineCaptor = ArgumentCaptor.forClass(Machine.class);
        ArgumentCaptor<String> stringCaptor = ArgumentCaptor.forClass(String.class);

        mockMvc.perform(post("/rest/machines")
                .header(HttpHeaders.AUTHORIZATION, TEST_HEADER)
                .contentType(MediaType.APPLICATION_JSON_VALUE)
                .content(gson.toJson(newMachine)))
                .andDo(print())
                .andExpect(status().isCreated());

        verify(connectService).createMachine(machineCaptor.capture(), stringCaptor.capture());

        assertEquals("SchwartenLeger", machineCaptor.getValue().getName());
    }

    @Test
    public void postPersonRequestTest() throws Exception {
        var newPerson = new Person();
        newPerson.setName("Otto");
        Gson gson = new Gson();

        when(connectService.createPerson(any(Person.class), anyString())).thenReturn(newPerson);
        ArgumentCaptor<Person> personCaptor = ArgumentCaptor.forClass(Person.class);
        ArgumentCaptor<String> stringCaptor = ArgumentCaptor.forClass(String.class);

        mockMvc.perform(post("/rest/persons")
                .header(HttpHeaders.AUTHORIZATION, TEST_HEADER)
                .contentType(MediaType.APPLICATION_JSON_VALUE)
                .content(gson.toJson(newPerson)))
                .andDo(print())
                .andExpect(status().isCreated());

        verify(connectService).createPerson(personCaptor.capture(), stringCaptor.capture());

        assertEquals("Otto", personCaptor.getValue().getName());
    }
}
