package com.farmnet365.hello365.integrationtests;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */


import com.farmnet365.hello365.model.ErrorDto;
import com.farmnet365.hello365.model.machine.Machine;
import com.farmnet365.hello365.model.machine.MachineResponse;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.tomakehurst.wiremock.core.Options;
import com.github.tomakehurst.wiremock.junit.WireMockRule;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.http.HttpStatus;
import org.springframework.http.RequestEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.junit4.SpringRunner;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import static com.farmnet365.hello365.integrationtests.TestJWTFactory.createInvalidJwt;
import static com.farmnet365.hello365.integrationtests.TestJWTFactory.createValidJwt;
import static com.github.tomakehurst.wiremock.client.WireMock.*;
import static org.assertj.core.api.Assertions.assertThat;

@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class GetMachinesIT {

	@Autowired
	private TestRestTemplate client;

	@Autowired
	private ObjectMapper objectMapper;

	@Rule
	public WireMockRule wireMockRule = new WireMockRule(Options.DYNAMIC_PORT);

	@Test
	public void successful() throws Exception {
		//GIVEN: 365Farmnet-Connect responses as expected
		MachineResponse machineResponse = createMachineResponse();
		stubFor(get(urlPathMatching("/connect/v1/machines")).withHeader("Authorization", matching(".*"))
				.willReturn(aResponse()
						.withStatus(200)
						.withHeader("Content-Type", "application/json")
						.withBody(objectMapper.writeValueAsString(machineResponse))));
		//AND: valid token with wiremock-server as api-base
		String generatedToken = createValidJwt(wireMockRule.baseUrl());

		//WHEN: requesting machines at hello
		RequestEntity requestEntity = RequestEntity.get(URI.create("/rest/machines")).header("Authorization", generatedToken).build();
		ResponseEntity<MachineResponse> responseEntity = client.exchange(requestEntity, MachineResponse.class);

		//THEN: Response is ok
		assertThat(responseEntity.getStatusCode()).isEqualTo(HttpStatus.OK);
		assertThat(responseEntity.getBody()).isEqualTo(machineResponse);
	}

	@Test
	public void failsWhenTokenInvalid() throws Exception {
		//GIVEN: 365Farmnet-Connect responses as expected
		MachineResponse machineResponse = createMachineResponse();
		stubFor(get(urlPathMatching("/connect/v1/machines")).withHeader("Authorization", matching(".*"))
				.willReturn(aResponse()
						.withStatus(200)
						.withHeader("Content-Type", "application/json")
						.withBody(objectMapper.writeValueAsString(machineResponse))));
		//AND: invalid token
		String generatedToken = createInvalidJwt(wireMockRule.baseUrl());

		//WHEN: requesting machines at hello
		RequestEntity requestEntity = RequestEntity.get(URI.create("/rest/machines")).header("Authorization", generatedToken).build();
		ResponseEntity<ErrorDto> responseEntity = client.exchange(requestEntity, ErrorDto.class);

		//THEN: response is unauthorized
		assertThat(responseEntity.getStatusCode()).isEqualTo(HttpStatus.UNAUTHORIZED);
		assertThat(responseEntity.getBody()).isNotNull();
		assertThat(responseEntity.getBody().getCode()).isEqualTo("0004_SignatureVerificationException");
	}

	@Test
	public void failsWhenConnectApiNotAccessible() throws Exception {
		//GIVEN: 365Farmnet-Connect responses as expected
		MachineResponse machineResponse = createMachineResponse();
		stubFor(get(urlPathMatching("/connect/v1/machines")).withHeader("Authorization", matching(".*"))
				.willReturn(aResponse()
						.withStatus(404)
						.withHeader("Content-Type", "application/json")));
		//AND: valid token with wiremock-server as api-base
		String generatedToken = createValidJwt(wireMockRule.baseUrl());

		//WHEN: requesting machines at hello
		RequestEntity requestEntity = RequestEntity.get(URI.create("/rest/machines")).header("Authorization", generatedToken).build();
		ResponseEntity<ErrorDto> responseEntity = client.exchange(requestEntity, ErrorDto.class);

		//THEN: response is 500
		assertThat(responseEntity.getStatusCode()).isEqualTo(HttpStatus.INTERNAL_SERVER_ERROR);
		assertThat(responseEntity.getBody()).isNotNull();
		assertThat(responseEntity.getBody().getCode()).isEqualTo("0003_HttpClientError");
	}

	private MachineResponse createMachineResponse() {
		Machine machine1 = new Machine();
		machine1.setId(UUID.randomUUID().toString());
		machine1.setName("My Tractor");
		Machine machine2 = new Machine();
		machine2.setId(UUID.randomUUID().toString());
		machine2.setName("My Harvester");
		List<Machine> machineList = new ArrayList<>();
		machineList.add(machine1);
		MachineResponse machineResponse = new MachineResponse();
		machineResponse.setList(machineList);
		return machineResponse;
	}


}
