package com.farmnet365.hello365.integrationtests;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import com.farmnet365.hello365.clients.RestClient;
import com.farmnet365.hello365.model.field.Field;
import com.farmnet365.hello365.model.machine.Machine;
import com.farmnet365.hello365.model.machine.MachineResponse;
import com.farmnet365.hello365.model.person.Person;
import com.farmnet365.hello365.model.person.PersonResponse;
import com.github.tomakehurst.wiremock.core.Options;
import com.github.tomakehurst.wiremock.junit.WireMockRule;
import com.google.gson.Gson;
import org.junit.Rule;
import org.junit.Test;

import java.util.List;

import static com.github.tomakehurst.wiremock.client.WireMock.*;
import static org.junit.Assert.*;

/**
 * This class shows how wiremock can be used to test the RestClient.
 */
public class RestClientIT {

    private static final String APPLICATION_JSON = "application/json";
    private static final String PARTNER_TOKEN = "tttttttteeeeeeessssssssstttttt";

    @Rule
    public WireMockRule wireMockRule = new WireMockRule(Options.DYNAMIC_PORT);

    @Test
    public void getFieldsEmpty() {

        stubFor(get(urlPathMatching("/connect/v1/fields"))
                .willReturn(aResponse()
                        .withStatus(200)
                        .withHeader("Content-Type", APPLICATION_JSON)
                        .withBody("[]")));

        RestClient restClient = new RestClient();
        List<Field> result = restClient.getFields(wireMockRule.baseUrl(), PARTNER_TOKEN);

        verify(getRequestedFor(urlEqualTo("/connect/v1/fields")).withHeader("Authorization", matching(PARTNER_TOKEN)));
        assertEquals(0, result.size());
    }

    @Test
    public void getPersons() {

        Person person = new Person();
        person.setName("Hugo Setzer");
        PersonResponse response = new PersonResponse(List.of(person));
        Gson gson = new Gson();

        stubFor(get(urlPathMatching("/connect/v1/persons"))
                .willReturn(aResponse()
                        .withStatus(200)
                        .withHeader("Content-Type", APPLICATION_JSON)
                        .withBody(gson.toJson(response))));

        RestClient restClient = new RestClient();
        PersonResponse result = restClient.getPersons(wireMockRule.baseUrl(), PARTNER_TOKEN);

        verify(getRequestedFor(urlEqualTo("/connect/v1/persons")).withHeader("Authorization", matching(PARTNER_TOKEN)));
        assertNotNull(result);
        assertNotNull(result.getList());
        assertEquals(1, result.getList().size());
        assertEquals("Hugo Setzer", result.getList().get(0).getName());
    }

    @Test
    public void getMachines() {

        Machine machine = new Machine();
        machine.setName("Egge");
        MachineResponse response = new MachineResponse(List.of(machine));
        Gson gson = new Gson();

        stubFor(get(urlPathMatching("/connect/v1/machines"))
                .willReturn(aResponse()
                        .withStatus(200)
                        .withHeader("Content-Type", APPLICATION_JSON)
                        .withBody(gson.toJson(response))));

        RestClient restClient = new RestClient();
        MachineResponse result = restClient.getMachines(wireMockRule.baseUrl(), PARTNER_TOKEN);

        verify(getRequestedFor(urlEqualTo("/connect/v1/machines")).withHeader("Authorization", matching(PARTNER_TOKEN)));
        assertNotNull(result);
        assertNotNull(result.getList());
        assertEquals(1, result.getList().size());
        assertEquals("Egge", result.getList().get(0).getName());
    }


    @Test
    public void postPerson() {

        stubFor(post(urlPathMatching("/connect/v1/persons"))
                .willReturn(aResponse()
                        .withStatus(201)
                        .withHeader("Content-Type", APPLICATION_JSON)
                        .withBody("{ \"id\": \"3475000\"}")));

        RestClient restClient = new RestClient();

        Person person = new Person();
        person.setName("Tester");
        Person result = restClient.postPerson(wireMockRule.baseUrl(), PARTNER_TOKEN, person);

        verify(postRequestedFor(urlEqualTo("/connect/v1/persons")).withHeader("Authorization", matching(PARTNER_TOKEN)));
        assertNotNull(result);
        assertEquals("3475000", result.getId());
    }

    @Test
    public void postMachine() {

        stubFor(post(urlPathMatching("/connect/v1/machines"))
                .willReturn(aResponse()
                        .withStatus(201)
                        .withHeader("Content-Type", APPLICATION_JSON)
                        .withBody("{ \"id\": \"7744\"}")));

        RestClient restClient = new RestClient();

        Machine machine = new Machine();
        machine.setName("xxTTzz");
        Machine result = restClient.postMachine(wireMockRule.baseUrl(), PARTNER_TOKEN, machine);

        verify(postRequestedFor(urlEqualTo("/connect/v1/machines")).withHeader("Authorization", matching(PARTNER_TOKEN)));
        assertNotNull(result);
        assertEquals("7744", result.getId());
    }
}
