package com.farmnet365.hello365.integrationtests;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */


import io.jsonwebtoken.Clock;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import io.jsonwebtoken.impl.DefaultClock;
import org.apache.commons.io.IOUtils;
import org.bouncycastle.jcajce.spec.OpenSSHPrivateKeySpec;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.util.io.pem.PemObject;
import org.bouncycastle.util.io.pem.PemReader;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.Security;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public class TestJWTFactory {
	private static final String VALID_TEST_PRIVATE_KEY = "-----BEGIN RSA PRIVATE KEY-----\n"
			+ "MIIEpAIBAAKCAQEAq/4rEV5TmWNdV55ItVXLTrz889YvvyX+5Fle0o/GZmfLDcT0\n"
			+ "IyG1fPSwImreASXBoppsiIW3l16FqKcr0tqNkQJqb5jkXY8WAKfN7QfzX3O6Dnjw\n"
			+ "/VwYLjFSCy39YrzEbSReoSj5PU/b6BQOUzohSaYf2nwPTTSKDKwZWew1oQycxCXH\n"
			+ "x34Xk9x69+iOGbst3mlppRjiWTOtbvxlmTJzpFf3u/NLD1lBk+5qYIl/3459yFiL\n"
			+ "79mXRXaVQteWrp93jNtdsZE+U2iF1VmQ3TqjbSVvLregpeo46mTb25kqeP7z6i18\n"
			+ "CoO6plcrjJ3D+/xak2DxLHXHZQEFs62GjdoR2QIDAQABAoIBAAQNUOZv2nhapvBR\n"
			+ "glUDWG7adPspqKZQNtkaYemECaCQAXCpmzbYrHXtLdkhu2k7gnxaNwRgYhrm1y9K\n"
			+ "APZOv9cSv38xku/BVztcuLVE4q71AwUkREkzyj4MW2p4Aug4mkgX+0LAvF59ayjk\n"
			+ "kBHV5MLfbxCuq42eF0KFK3O65OiOYZ0e88X+2gjCKp7uvHmTJFWSZEUirWiP5VCn\n"
			+ "jFhuihg1qJBo1lf+gnEoHHZlToxNop4uZPvzKw23N73VJlDwGTuwlG3uknWJSGrA\n"
			+ "LZ4D4W8fzoFr1u0i4nTyF2rTjtJN/qaKa0geG2hLIWQVGXEJXdMV248VPHUQAj/N\n"
			+ "tfp/xaECgYEA3BOffqpoTj7DukSiiSbmRpUCwm62hL5zTw3zJoOdUBBtfo7S5CRi\n"
			+ "ab42GCVXXM9RncVmIuovKsWja4glMMqY+tmN84jA9AvkxU0aPGpITR/db5STXiDy\n"
			+ "RCCLy7F1+G5I27SmEY5l4uSDjNmAjl62qL9UUgmbhd/tqNsz5wEygOcCgYEAyBFC\n"
			+ "fVmqV1MS4wFOvUUiLsuaFA7EGWtydinYqoctQksA5O/g8Lpogv++BYvQfP5M+JSv\n"
			+ "kG0aAj0ghGpM5t77hAfkm3vheAXmPzNB6/YDrAVy1keDtG4efXA5KzRBbhXZv+m+\n"
			+ "0toP/n0YUJI0K0/fv65Mg8nZq0Cv/rNXgQHAvz8CgYEArMWS/yPUwyivC/p6B2ZM\n"
			+ "fbjZ875q94d0iumOq/vDcmmTLDoAAi82O4Gigg4+qZPwpQWli1+CK/gAtZiqacWD\n"
			+ "l9tXf4sqv+zeaq5/HFLr8/BCLSjdEJaY2zcZyI3G6YPbGvCVN08tWZTAWcEfFaXx\n"
			+ "jES4c7crK0yDDX8ZHg9okJ0CgYBv195ArwigCqvrRYbcHjLO0DlRbzPHNh7l2dzq\n"
			+ "7+ioP7+/PC6WxM66NPfop9GknpEvAgfniGZazAtKmxZkllEq7TKqjSVtcHbX6/K1\n"
			+ "U3GYD++zxITnMAxReZQsB+lv/j0doTFxNEBut0YJfMI3WE6wHGNf5U6PAvmgPGf1\n"
			+ "kaJMBwKBgQDbcU9iFZPo5yKyXrcKo3SHI3lhoiRWq85VgYtlJDJB639T7rKOVy/a\n"
			+ "klwrDHHKCRdLx640WQAsK43eOKdH61vdyg8pU1SZd/wK7VuW3fHQK3kh1jq/nCvN\n"
			+ "5VxAnLPy5xSnjd+UJKQ6XsO14ngf2fw2OfTuCu05/7hnzjkgjLmMWQ==\n" +
			"-----END RSA PRIVATE KEY-----";

	private static final String INVALID_TEST_PRIVATE_KEY = "-----BEGIN RSA PRIVATE KEY-----\n"
			+ "MIIEogIBAAKCAQEAmtSvYCbMmxjcDmuVoNI9qvAu4KTJKrEMwVreLit2dmCxeKh+\n"
			+ "j1h0bc34U3Vlz0+qmALjyDhjzgPGbjOVLR569+2YS2H9yUfW3vURl0Gt4xNhQyoh\n"
			+ "5PvH3YDZ6Y9MxNu7vuluIHoChdKayrkiG5q506MB7tugp6qeT7MHNCTwC5bMlXRU\n"
			+ "X2mClbrzGwEhFrB0RRnhCxbqcbVN1msTFt/ByXA7OhmRwpQPaHpAxBNmUN85ZDrC\n"
			+ "SWTE9p40N4Gxfco4SYWVF8laY8aypVYONRs56RM8seKjWPD1p73Uw9tncRW1cfcl\n"
			+ "3k6t87snQSJIZrCGU+uvComp4CtpDXoQ7cZHzQIDAQABAoIBAFz20w5SfYuqk4Z9\n"
			+ "Bmx3j9txN1Re3QKrJ9N48eqa4hgV2rltwf/jxpSflTjXRngBh5aaNZZkdVifX6eP\n"
			+ "hUSABqmSmVy5y53tapCYaSV+pB+x5TZkt9E7dDgFQ9I+3oNcxA/PLPaHC7i6SYLr\n"
			+ "QhV9A5KVT3XRnVbO1RtZjTu76f2qcLtbeFCljpwRdec443V4w66+okfbI1Y943Ou\n"
			+ "XhztVTnaa4MlmhIhkHk1VJpr/kk8ya2ufmtR9ZXQx3Inl90fT8K7tNwMICQ3fqh7\n"
			+ "tjvAxRSm5iz2zIflVNKleqLDr/rxpZrjACeiJ0Hu6juytdsT4i0sfyFau6UIkXy+\n"
			+ "GSGyMGkCgYEAx90Byxz0JIbkFBo6GYbax+5xT8jmapHp5E8WU88Jlpe4cTbikE6o\n"
			+ "Ba4l/y/DeP2c4UWBoga+/xFN5pMkV2t7KxELEOf0QcwGNSj+8eWVmbpkAMhueTBT\n"
			+ "RNRToJoNztXLZrljIiMdKwkYI+wq7F+mnGGgG7A2h5UrlQfkeQSVh0MCgYEAxlGn\n"
			+ "30cGvGtZpjQkNofiGshjrZ7LrdxTWD0+vwZkhp4rqPq2De658KZYRyZcmHg7DUZy\n"
			+ "KTaiskn7FUvkpQaXQmXnrAQ2L6jqlukCZsdPzhPUO7EOTSr8JinBvIIwDH7PhyyV\n"
			+ "jmCAbtbfC/RZppjwCflHKalyu3O8m5UrEdD+W68CgYA+j11aWKvLKzYFXNhG2UFu\n"
			+ "R6LMWJRXU2kk/+R7Q1k6/C7NsWJlzVkmvgZCxI6pNql+kiUrtHTgnmgF/pTnkEsz\n"
			+ "hdgy6Ig6eESRK03yJe0X+5IF9Lcv1AKcjuHGabJcxesvMmRncRpWODiINgqRhrc+\n"
			+ "SAi34gaPlg/YTGolv1uWxQKBgA867CJ4iOrlb6AU9wb9Dn8yDn2h6VBz0C7BnVT8\n"
			+ "P7Ia6Y1zm5GcWMlDbRohtxC6gwZ094V0va/lgJPVHT0TMYCW5ODP/f1RApsOT/Hm\n"
			+ "splH6o+e/FMuzuh7YnaApQIApx1hbibeAF8Cv2RiopPpwN1zOGTZr4b3z28/WGxo\n"
			+ "2W/fAoGABfOCS68ZUIg77iE4XGVl3pUv+122VM0NxjjaclFewfr5LCWtqCy+qWeS\n"
			+ "Tn+zjJ4na8g2oPF1ntg0AbwUtEs+Bw5gUlfnwx5GwV8qlyGYO40t0nGNFehcKNIx\n"
			+ "lzkDZP388CMTk587YjIZGNOsCPjzMXrmpn4CqxOz2uv+FWB1JII=\n" +
			"-----END RSA PRIVATE KEY-----\n";

	public static String createValidJwt(String apiBase) throws Exception {
		return createJwt(apiBase, VALID_TEST_PRIVATE_KEY, 60000);
	}

	public static String createInvalidJwt(String apiBase) throws Exception {
		return createJwt(apiBase, INVALID_TEST_PRIVATE_KEY, 60000);
	}

	public static String createExpiredJwt(String apiBase) throws Exception {
		return createJwt(apiBase, VALID_TEST_PRIVATE_KEY, -60000);
	}

	private static String createJwt(String apiBase, String privateKeyString, int expirationOffset) throws Exception {
		PrivateKey privateKey = readPrivateKeyFromPemString(privateKeyString);
		Clock clock = new DefaultClock();
		return Jwts.builder()
				.setHeader(buildHeaders())
				.setClaims(buildClaims(apiBase))
				.setIssuedAt(clock.now())
				.setExpiration(new Date(clock.now().getTime() + expirationOffset))
				.setNotBefore(clock.now())
				.signWith(SignatureAlgorithm.RS256, privateKey).compact();
	}

	private static Map<String, Object> buildHeaders() {
		final Map<String, Object> headers = new HashMap<>();
		headers.put("ver", "0.2");
		headers.put("type", "connect");
		return headers;
	}

	private static Map<String, Object> buildClaims(String apiBase) {
		final Map<String, Object> claims = new HashMap<>();
		claims.put("fn-ext", Collections.singletonMap("apiBase", apiBase));
		return claims;
	}

	private static PrivateKey readPrivateKeyFromPemString(String privateKey) throws Exception {
		PemObject pemObject = readPemObject(privateKey);
		OpenSSHPrivateKeySpec privateKeySpec = new OpenSSHPrivateKeySpec(pemObject.getContent());
		KeyFactory factory = KeyFactory.getInstance("RSA", "BC");
		return factory.generatePrivate(privateKeySpec);
	}

	private static PemObject readPemObject(String privateKey) throws IOException {
		Security.addProvider(new BouncyCastleProvider());

		InputStream inputStream = IOUtils.toInputStream(privateKey, "UTF-8");
		try (PemReader pemReader = new PemReader(new InputStreamReader(inputStream))) {
			return pemReader.readPemObject();
		}
	}
}
