package com.farmnet365.hello365.service;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */


import com.auth0.jwt.JWT;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTDecodeException;
import com.auth0.jwt.interfaces.Claim;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.farmnet365.hello365.config.ApiProperties;
import com.farmnet365.hello365.exception.JwtVerifyException;
import io.jsonwebtoken.JwtBuilder;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.apache.commons.lang3.StringUtils;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;

import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.*;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class JwtServiceTest {

    private static final String testSecret = "11111111111111111111111111";
    private static final String testID = "22222222222222222222222222";
    private static final String FN_EXT_CLAIM = "fn-ext";
    private static final String API_BASE = "http://localhost/my/test/api/base";
    private static final String BEARER = "Bearer ";
    private static final String CON = "con";


    @Mock
    private ApiProperties properties;

    @Mock
    private SigningAlgorithmService signingAlgorithmService;

    @InjectMocks
    private JwtService jwtService;

    @Before
    public void setup() {
        when(properties.getId()).thenReturn(testID);
        when(signingAlgorithmService.getPartnerTokenAlgorithm()).thenReturn(Algorithm.HMAC256(testSecret));
    }

    /**
     * Test decode.
     */
    @Test
    public void testDecodeConnectJwt() {
        LocalDate now = LocalDate.now();
        LocalDate later = now.plusDays(5);
        String token = makeValidTestToken(now, later);

        DecodedJWT decodedJwt = jwtService.decodeConnectJwt(token);

        assertNotNull(decodedJwt);
        assertFalse(decodedJwt.getClaim(FN_EXT_CLAIM).isNull());
        assertEquals(decodedJwt.getIssuedAt().toInstant(), now.atStartOfDay(ZoneId.systemDefault()).toInstant());
        assertEquals(decodedJwt.getExpiresAt().toInstant(), later.atStartOfDay(ZoneId.systemDefault()).toInstant());
    }

    @Test(expected = JwtVerifyException.class)
    public void testInvalidDecodeConnectJwt1() {
        jwtService.decodeConnectJwt(null);
    }

    @Test(expected = JwtVerifyException.class)
    public void testInvalidDecodeConnectJwt2() {
        jwtService.decodeConnectJwt("fooBar");
    }

    /**
     * Test verify of valid JWT
     */
    @Test
    public void testVerifyConnectJwt() {
        LocalDate now = LocalDate.now();
        LocalDate later = now.plusDays(5);
        String token = makeValidTestToken(now, later);

        DecodedJWT decodedJwt = jwtService.decodeConnectJwt(token);
        jwtService.verifyConnectJwt(decodedJwt);
    }

    /**
     * Test verify of invalid JWT
     */
    @Test(expected = JwtVerifyException.class)
    public void testVerifyInvalidConnectJwt() {
        LocalDate now = LocalDate.now();
        LocalDate later = now.plusDays(5);
        String token = makeTestTokenWithoutFnExt(now, later);

        DecodedJWT decodedJwt = jwtService.decodeConnectJwt(token);
        jwtService.verifyConnectJwt(decodedJwt);
    }

    @Test(expected = JwtVerifyException.class)
    public void testExpiredConnectJwt() {
        String token = makeExpiredTestToken();

        DecodedJWT decodedJwt = jwtService.decodeConnectJwt(token);
        jwtService.verifyConnectJwt(decodedJwt);
    }

    @Test(expected = JwtVerifyException.class)
    public void testInvalidIssuedAtConnectJwt() {
        String token = makeFutureTestToken();

        DecodedJWT decodedJwt = jwtService.decodeConnectJwt(token);
        jwtService.verifyConnectJwt(decodedJwt);
    }

    @Test(expected = JwtVerifyException.class)
    public void testWithoutApiBaseConnectJwt() {
        String token = makeTestTokenWithoutApiBase();

        DecodedJWT decodedJwt = jwtService.decodeConnectJwt(token);
        jwtService.verifyConnectJwt(decodedJwt);
    }

    /**
     * Test creation of new partner JWT
     */
    @Test
    public void testCreatePartnerToken() {
        LocalDate now = LocalDate.now();
        LocalDate later = now.plusDays(5);
        String token = makeValidTestToken(now, later);

        DecodedJWT decodedJwt = jwtService.decodeConnectJwt(token);
        String base = jwtService.extractApiBase(decodedJwt);
        assertEquals(API_BASE, base);

        ConnectApiAuthenticationInfo partnerTokenInfo = jwtService.createPartnerJwt(token, decodedJwt, API_BASE);

        assertNotNull(partnerTokenInfo);
        assertEquals(partnerTokenInfo.getApiBase(), API_BASE);
        assertNotNull(partnerTokenInfo.getPartnerToken());
        assertTrue(partnerTokenInfo.getPartnerToken().startsWith(BEARER));

        DecodedJWT claims = decodeConnectJwt(StringUtils.removeStart(partnerTokenInfo.getPartnerToken(), BEARER));
        Claim con = claims.getClaim(CON);
        assertFalse(con.isNull());
        assertEquals(con.asString(), token);
    }

    @Test
    public void testGetPartnerToken() {
        ConnectApiAuthenticationInfo partnerToken = jwtService.generatePartnerJwt(makeValidTestToken());
        assertNotNull(partnerToken);
    }


    // ----------- helper-methods -----------
    private String makeValidTestToken() {
        LocalDate now = LocalDate.now();
        LocalDate later = now.plusDays(5);
        return makeValidTestToken(now, later);
    }

    private String makeExpiredTestToken() {
        LocalDate now = LocalDate.now();
        LocalDate later = now.minusDays(1);
        return makeValidTestToken(now, later);
    }

    private String makeFutureTestToken() {
        LocalDate now = LocalDate.now().plusDays(2);
        LocalDate later = now.plusDays(10);
        return makeValidTestToken(later, now);
    }

    private String makeValidTestToken(LocalDate now, LocalDate later) {

        JwtBuilder builder = Jwts.builder()
                .setHeader(generatePartnerJwtHeader())
                .setIssuer(testID)
                .setExpiration(Date.from(later.atStartOfDay(ZoneId.systemDefault()).toInstant()))
                .setIssuedAt(Date.from(now.atStartOfDay(ZoneId.systemDefault()).toInstant()))
                .claim(FN_EXT_CLAIM, new FnExt(API_BASE))
                .signWith(SignatureAlgorithm.HS256, testSecret);

        return builder.compact();
    }

    private String makeTestTokenWithoutFnExt(LocalDate now, LocalDate later) {

        JwtBuilder builder = Jwts.builder()
                .setHeader(generatePartnerJwtHeader())
                .setIssuer(testID)
                .setExpiration(Date.from(later.atStartOfDay(ZoneId.systemDefault()).toInstant()))
                .setIssuedAt(Date.from(now.atStartOfDay(ZoneId.systemDefault()).toInstant()))
                .signWith(SignatureAlgorithm.HS256, testSecret);

        return builder.compact();
    }

    private String makeTestTokenWithoutApiBase() {
        LocalDate now = LocalDate.now();
        LocalDate later = now.plusDays(5);

        JwtBuilder builder = Jwts.builder()
                .setHeader(generatePartnerJwtHeader())
                .setIssuer(testID)
                .setExpiration(Date.from(later.atStartOfDay(ZoneId.systemDefault()).toInstant()))
                .setIssuedAt(Date.from(now.atStartOfDay(ZoneId.systemDefault()).toInstant()))
                .claim(FN_EXT_CLAIM, new FnExt())
                .signWith(SignatureAlgorithm.HS256, testSecret);

        return builder.compact();
    }

    private Map<String, Object> generatePartnerJwtHeader() {
        Map<String, Object> jwtHeader = new HashMap<>();
        jwtHeader.put("ver", "0.1");
        jwtHeader.put("type", "partner");
        jwtHeader.put("alg", "HS256");
        return jwtHeader;
    }

    private DecodedJWT decodeConnectJwt(final String connectJwt) {
        if (connectJwt == null) {
            throw new JwtVerifyException("No connect jwt present.");
        }
        try {
            return JWT.decode(connectJwt);
        } catch (JWTDecodeException ex) {
            throw new JwtVerifyException("Unable to decode connect jwt.", ex);
        }
    }

    static class FnExt {
        private String apiBase;

        FnExt(String apiBase) {
            this.apiBase = apiBase;
        }

        public FnExt() {
        }

        public String getApiBase() {
            return apiBase;
        }

        public void setApiBase(String apiBase) {
            this.apiBase = apiBase;
        }
    }
}
